/**
*
* La v0.2 accende e spegne (toggle) le 5 prese alla pressione dei tasti da 1 a 5 sul telecomando bravo B124
*
* La v0.3 ha in più il controllo nello startup dei nuovi pulsanti D2 e D3 per accedere al setup via TV
* la struttura dei menu e lo scheletro del sistema di lettura dei codici del telecomando
*
* La v0.4 ha in più il salvataggio/ripristino dei codici telecomando su eeprom
*
* La v0.5 utilizza unsigned long per salvare la configurazione dei tasti perchè alcuni protocolli utilizzano 
* fino a 32 bits per codificare i tasti e implementa il salvataggio e ripristino configurazione delle prese
* dopo blackout
*/

#include <EEPROM.h>
#include <IRremote.h>
#include <TVout.h>
#include <fontALL.h>

/**
*  Variabili globali ricevitore IR
*/
int RECV_PIN = 11;
IRrecv irrecv(RECV_PIN);
decode_results results;

/**
*  Variabili globali TV out
*/
TVout TV;

/**
*  Variabili globali prese (Stato e appartenenza gruppi)
*/
byte pinStatus[5] = {0,0,0,0,0};
byte group1[5] = {0,0,0,0,0};
byte group2[5] = {0,0,0,0,0};

/**
*  Variabili globali configurazione tasti telecomando
*/
unsigned long  irKeysToggle[8] = {0L,0L,0L,0L,0L,0L,0L,0L};
unsigned long  irKeysOn[8]     = {0L,0L,0L,0L,0L,0L,0L,0L};
unsigned long  irKeysOff[8]    = {0L,0L,0L,0L,0L,0L,0L,0L};

/**
*  Variabili globali gestione fase di avvio
*/
byte restartMode = 5;

/**
*  writeLong: scrive un valore long (32 bits) all'indirizzo dato della EEPROM
*/
void writeLong(byte address, long value)
{
  EEPROM.write(address+3,lowByte(value));
  EEPROM.write(address+2,lowByte(value>>8));
  EEPROM.write(address+1,lowByte(value>>16));
  EEPROM.write(address,lowByte(value>>24));
}

/**
*  readLong: legge un valore long (32 bits) dall'inirizzo dato della EEPROM
*/
long readLong(byte address)
{
  long ret;
  //ret = EEPROM.read(address+3);
  //ret += (EEPROM.read(address+2)<<8);
  //ret += (EEPROM.read(address+1)<<16);
  //ret += (EEPROM.read(address)<<24);
  ret = EEPROM.read(address);
  ret = ret << 8;
  ret += EEPROM.read(address+1);
  ret = ret << 8;
  ret += EEPROM.read(address+2);
  ret = ret << 8;
  ret += EEPROM.read(address+3);
  
  return ret;
}

/**
*  saveToEEprom: salva i parametri di configurazione nella EEPROM
*/
void saveToEEprom()
{
  writeLong(10,irKeysToggle[0]);
  writeLong(14,irKeysToggle[1]);
  writeLong(18,irKeysToggle[2]);
  writeLong(22,irKeysToggle[3]);
  writeLong(26,irKeysToggle[4]);
  writeLong(30,irKeysToggle[5]);
  writeLong(34,irKeysToggle[6]);
  writeLong(38,irKeysToggle[7]);

  writeLong(42,irKeysOn[0]);
  writeLong(46,irKeysOn[1]);
  writeLong(50,irKeysOn[2]);
  writeLong(54,irKeysOn[3]);
  writeLong(58,irKeysOn[4]);
  writeLong(62,irKeysOn[5]);
  writeLong(66,irKeysOn[6]);
  writeLong(70,irKeysOn[7]);

  writeLong(74,irKeysOff[0]);
  writeLong(78,irKeysOff[1]);
  writeLong(82,irKeysOff[2]);
  writeLong(86,irKeysOff[3]);
  writeLong(90,irKeysOff[4]);
  writeLong(94,irKeysOff[5]);
  writeLong(98,irKeysOff[6]);
  writeLong(102,irKeysOff[7]);

  EEPROM.write(108, restartMode);
  
  EEPROM.write(110, group1[0]);
  EEPROM.write(111, group1[1]);
  EEPROM.write(112, group1[2]);
  EEPROM.write(113, group1[3]);
  EEPROM.write(114, group1[4]);

  EEPROM.write(115, group2[0]);
  EEPROM.write(116, group2[1]);
  EEPROM.write(117, group2[2]);
  EEPROM.write(118, group2[3]);
  EEPROM.write(119, group2[4]);
  
}

/**
*  readFromEEprom: Legge i parametri di configurazione dalla EEPROM
*/
void readFromEEprom()
{
  irKeysToggle[0] = readLong(10);
  irKeysToggle[1] = readLong(14);
  irKeysToggle[2] = readLong(18);
  irKeysToggle[3] = readLong(22);
  irKeysToggle[4] = readLong(26);
  irKeysToggle[5] = readLong(30);
  irKeysToggle[6] = readLong(34);
  irKeysToggle[7] = readLong(38);

  irKeysOn[0] = readLong(42);
  irKeysOn[1] = readLong(46);
  irKeysOn[2] = readLong(50);
  irKeysOn[3] = readLong(54);
  irKeysOn[4] = readLong(58);
  irKeysOn[5] = readLong(62);
  irKeysOn[6] = readLong(66);
  irKeysOn[7] = readLong(70);
  
  irKeysOff[0] = readLong(74);
  irKeysOff[1] = readLong(78);
  irKeysOff[2] = readLong(82);
  irKeysOff[3] = readLong(86);
  irKeysOff[4] = readLong(90);
  irKeysOff[5] = readLong(94);
  irKeysOff[6] = readLong(98);
  irKeysOff[7] = readLong(102);
  
  restartMode = EEPROM.read(108);
  
  group1[0] = EEPROM.read(110);
  group1[1] = EEPROM.read(111);
  group1[2] = EEPROM.read(112);
  group1[3] = EEPROM.read(113);
  group1[4] = EEPROM.read(114);

  group2[0] = EEPROM.read(115);
  group2[1] = EEPROM.read(116);
  group2[2] = EEPROM.read(117);
  group2[3] = EEPROM.read(118);
  group2[4] = EEPROM.read(119);
  
}

void clearEEprom()
{
  
}

void loadLastSocketConf()
{
  pinStatus[0] = EEPROM.read(120);
  pinStatus[1] = EEPROM.read(121);
  pinStatus[2] = EEPROM.read(122);
  pinStatus[3] = EEPROM.read(123);
  pinStatus[4] = EEPROM.read(124);
}

void saveLastSocketConf()
{
  byte tmpStatus;
  
  for(byte p=0;p<5;p++)
  {
    tmpStatus = EEPROM.read(120+p);
    if(pinStatus[p]!=tmpStatus)
    {
      EEPROM.write(120+p,pinStatus[p]);
    }
  }
  
  //EEPROM.write(120, pinStatus[0]);
  //EEPROM.write(121, pinStatus[1]);
  //EEPROM.write(122, pinStatus[2]);
  //EEPROM.write(123, pinStatus[3]);
  //EEPROM.write(124, pinStatus[4]);
}

void resetConfig()
{
  irKeysToggle[0] = 0L;
  irKeysToggle[1] = 0L;
  irKeysToggle[2] = 0L;
  irKeysToggle[3] = 0L;
  irKeysToggle[4] = 0L;
  irKeysToggle[5] = 0L;
  irKeysToggle[6] = 0L;
  irKeysToggle[7] = 0L;

  irKeysOn[0] = 0L;
  irKeysOn[1] = 0L;
  irKeysOn[2] = 0L;
  irKeysOn[3] = 0L;
  irKeysOn[4] = 0L;
  irKeysOn[5] = 0L;
  irKeysOn[6] = 0L;
  irKeysOn[7] = 0L;
  
  irKeysOff[0] = 0L;
  irKeysOff[1] = 0L;
  irKeysOff[2] = 0L;
  irKeysOff[3] = 0L;
  irKeysOff[4] = 0L;
  irKeysOff[5] = 0L;
  irKeysOff[6] = 0L;
  irKeysOff[7] = 0L;
  
  restartMode = 5;
  
  group1[0] = 0;
  group1[1] = 0;
  group1[2] = 0;
  group1[3] = 0;
  group1[4] = 0;

  group2[0] = 0;
  group2[1] = 0;
  group2[2] = 0;
  group2[3] = 0;
  group2[4] = 0;
}

void blinkLed(byte pin, byte flash=1, int len=250, int pause=250)
{
  for(int c=0;c<flash;c++)
  {
    digitalWrite(pin,HIGH);
    delay(len);
    digitalWrite(pin,LOW);
    delay(pause);
  }
}

int checkEnterMenu()
{
  TV.print(1,30,"ATTENDO ");
  for(int t=5;t>=0;t--)
  {
    TV.print(32,30,t);
    for(int p=0;p<50;p++)
    {
       TV.delay(3);
       if(digitalRead(2)==LOW)
         return true;
    }
  }
  return false; 
}

byte readKeys()
{
  byte ret = 0;
    if(digitalRead(2)==LOW)
      ret += 1;
    if(digitalRead(3)==LOW)
      ret += 2;
  return ret;
}

byte waitKeys()
{
  byte ret = 0;
  while(ret==0)
  {
    if(digitalRead(2)==LOW)
      ret += 1;
    if(digitalRead(3)==LOW)
      ret += 2;
  }    
  return ret;
}

void drawMenu(int m)
{ 
    TV.clear_screen();
    switch(m)
    {
      case 1:
        TV.printPGM(30,0,PSTR("MENU PRINCIPALE"));
        TV.printPGM(8,16,PSTR("DEFINIZIONE GRUPPI"));
        TV.printPGM(8,24,PSTR("TASTI TELECOMANDO TOGGLE"));
        TV.printPGM(8,32,PSTR("TASTI TELECOMANDO ON"));
        TV.printPGM(8,40,PSTR("TASTI TELECOMANDO OFF"));
        TV.printPGM(8,48,PSTR("INTERRUZIONE CORRENTE"));
        TV.printPGM(8,56,PSTR("---"));
        TV.printPGM(8,64,PSTR("RESET CONFIGURAZIONE"));
        TV.printPGM(8,72,PSTR("SALVA IN EEPROM"));
        TV.printPGM(8,80,PSTR("ESCI"));
        break;
        
      case 2:
        TV.printPGM(22,0,PSTR("DEFINIZIONE GRUPPI"));
        TV.printPGM(8,16,PSTR("GRUPPO 1"));
        TV.printPGM(8,24,PSTR("GRUPPO 2"));
        TV.printPGM(8,32,PSTR("---"));
        TV.printPGM(8,40,PSTR("---"));
        TV.printPGM(8,48,PSTR("---"));
        TV.printPGM(8,56,PSTR("---"));
        TV.printPGM(8,64,PSTR("---"));
        TV.printPGM(8,72,PSTR("---"));
        TV.printPGM(8,80,PSTR("INDIETRO"));
        break;
        
      case 3:
        TV.printPGM(12,0,PSTR("TASTI TELECOMANDO TOGGLE"));
        TV.printPGM(8,16,PSTR("PRESA 1"));
        TV.printPGM(8,24,PSTR("PRESA 2"));
        TV.printPGM(8,32,PSTR("PRESA 3"));
        TV.printPGM(8,40,PSTR("PRESA 4"));
        TV.printPGM(8,48,PSTR("PRESA 5"));
        TV.printPGM(8,56,PSTR("GRUPPO 1"));
        TV.printPGM(8,64,PSTR("GRUPPO 2"));
        TV.printPGM(8,72,PSTR("PRESE 1-5"));
        TV.printPGM(8,80,PSTR("INDIETRO"));
        
        
        TV.print(50,16,irKeysToggle[0]);
        TV.print(50,24,irKeysToggle[1]);
        TV.print(50,32,irKeysToggle[2]);
        TV.print(50,40,irKeysToggle[3]);
        TV.print(50,48,irKeysToggle[4]);
        TV.print(50,56,irKeysToggle[5]);
        TV.print(50,64,irKeysToggle[6]);
        TV.print(50,72,irKeysToggle[7]);
        
        break;
      case 4:
        TV.printPGM(20,0,PSTR("TASTI TELECOMANDO ON"));
        TV.printPGM(8,16,PSTR("PRESA 1"));
        TV.printPGM(8,24,PSTR("PRESA 2"));
        TV.printPGM(8,32,PSTR("PRESA 3"));
        TV.printPGM(8,40,PSTR("PRESA 4"));
        TV.printPGM(8,48,PSTR("PRESA 5"));
        TV.printPGM(8,56,PSTR("GRUPPO 1"));
        TV.printPGM(8,64,PSTR("GRUPPO 2"));
        TV.printPGM(8,72,PSTR("PRESE 1-5"));
        TV.printPGM(8,80,PSTR("INDIETRO"));
        
        
        TV.print(50,16,irKeysOn[0]);
        TV.print(50,24,irKeysOn[1]);
        TV.print(50,32,irKeysOn[2]);
        TV.print(50,40,irKeysOn[3]);
        TV.print(50,48,irKeysOn[4]);
        TV.print(50,56,irKeysOn[5]);
        TV.print(50,64,irKeysOn[6]);
        TV.print(50,72,irKeysOn[7]);
        
        break;
      case 5:
        TV.printPGM(20,0,PSTR("TASTI TELECOMANDO OFF"));
        TV.printPGM(8,16,PSTR("PRESA 1"));
        TV.printPGM(8,24,PSTR("PRESA 2"));
        TV.printPGM(8,32,PSTR("PRESA 3"));
        TV.printPGM(8,40,PSTR("PRESA 4"));
        TV.printPGM(8,48,PSTR("PRESA 5"));
        TV.printPGM(8,56,PSTR("GRUPPO 1"));
        TV.printPGM(8,64,PSTR("GRUPPO 2"));
        TV.printPGM(8,72,PSTR("PRESE 1-5"));
        TV.printPGM(8,80,PSTR("INDIETRO"));
        
        
        TV.print(50,16,irKeysOff[0]);
        TV.print(50,24,irKeysOff[1]);
        TV.print(50,32,irKeysOff[2]);
        TV.print(50,40,irKeysOff[3]);
        TV.print(50,48,irKeysOff[4]);
        TV.print(50,56,irKeysOff[5]);
        TV.print(50,64,irKeysOff[6]);
        TV.print(50,72,irKeysOff[7]);
        
        break;

      case 6:
        TV.printPGM(30,0,PSTR("CONF GRUPPO 1"));
        TV.printPGM(8,16,PSTR("PRESA 1"));
        TV.printPGM(8,24,PSTR("PRESA 2"));
        TV.printPGM(8,32,PSTR("PRESA 3"));
        TV.printPGM(8,40,PSTR("PRESA 4"));
        TV.printPGM(8,48,PSTR("PRESA 5"));
        TV.printPGM(8,56,PSTR("---"));
        TV.printPGM(8,64,PSTR("---"));
        TV.printPGM(8,72,PSTR("---"));
        TV.printPGM(8,80,PSTR("INDIETRO"));
        
        if(group1[0]==1)
          TV.print(50,16,"V");
        else
          TV.print(50,16,"X");
        
        if(group1[1]==1)
          TV.print(50,24,"V");
        else
          TV.print(50,24,"X");
        
        if(group1[2]==1)
          TV.print(50,32,"V");
        else
          TV.print(50,32,"X");
        
        if(group1[3]==1)
          TV.print(50,40,"V");
        else
          TV.print(50,40,"X");
        
        if(group1[4]==1)
          TV.print(50,48,"V");
        else
          TV.print(50,48,"X");
        
        break;
      case 7:
        TV.printPGM(30,0,PSTR("CONF GRUPPO 2"));
        TV.printPGM(8,16,PSTR("PRESA 1"));
        TV.printPGM(8,24,PSTR("PRESA 2"));
        TV.printPGM(8,32,PSTR("PRESA 3"));
        TV.printPGM(8,40,PSTR("PRESA 4"));
        TV.printPGM(8,48,PSTR("PRESA 5"));
        TV.printPGM(8,56,PSTR("---"));
        TV.printPGM(8,64,PSTR("---"));
        TV.printPGM(8,72,PSTR("---"));
        TV.printPGM(8,80,PSTR("INDIETRO"));

        if(group2[0]==1)
          TV.print(50,16,"V");
        else
          TV.print(50,16,"X");
        
        if(group2[1]==1)
          TV.print(50,24,"V");
        else
          TV.print(50,24,"X");
        
        if(group2[2]==1)
          TV.print(50,32,"V");
        else
          TV.print(50,32,"X");
        
        if(group2[3]==1)
          TV.print(50,40,"V");
        else
          TV.print(50,40,"X");
        
        if(group2[4]==1)
          TV.print(50,48,"V");
        else
          TV.print(50,48,"X");
        
        break;
      case 8:
        TV.printPGM(12,0,PSTR("INTERRUZIONE CORRENTE"));
        TV.printPGM(8,16,PSTR("ULTIMA CONFIGURAZIONE"));
        TV.printPGM(8,24,PSTR("GRUPPO 1 ACCESO"));
        TV.printPGM(8,32,PSTR("GRUPPO 2 ACCESO"));
        TV.printPGM(8,40,PSTR("TUTTO ACCESO"));
        TV.printPGM(8,48,PSTR("TUTTO SPENTO"));
        TV.printPGM(8,56,PSTR("---"));
        TV.printPGM(8,64,PSTR("---"));
        TV.printPGM(8,72,PSTR("---"));
        TV.printPGM(8,80,PSTR("INDIETRO"));
        
        TV.printPGM(100,8+restartMode*8,PSTR("V"));
        break;
    }
    TV.printPGM(0,16,PSTR(">"));
}

int processMenu()
{
    
    int exitLoop = false;
    int key = 0;
    int m=1;
    int f=1;
    byte h,l;
    
    drawMenu(m);

    while(!exitLoop)
    {
      key = waitKeys();
      if(key == 1)  //D2 -> ENTER
      {
        switch(m)
        {
          case 1:
            switch(f)
            {
              case 1:
                m=2;
                f=1;
                drawMenu(m);
                break;
              case 2:
                m=3;
                f=1;
                drawMenu(m);
                break;
              case 3:
                m=4;
                f=1;
                drawMenu(m);
                break;
              case 4:
                m=5;
                f=1;
                drawMenu(m);
                break;
              case 5:
                m=8;
                f=1;
                drawMenu(m);
                break;
              case 7:
                resetConfig();
                TV.printPGM(92,64,PSTR("OK!"));
                TV.delay(500);
                TV.printPGM(92,64,PSTR("   "));
                break;
              case 8:
                saveToEEprom();
                TV.printPGM(92,72,PSTR("OK!"));
                TV.delay(500);
                TV.printPGM(92,72,PSTR("   "));
                break;
              case 9:
                return false;
                break;
            }
            break;
          case 2:  // MENU 2
            switch(f)
            {
              case 1:  // GRUPPO 1
                m=6;
                f=1;
                drawMenu(m);
                break;
              case 2:  // GRUPPO 2
                m=7;
                f=1;
                drawMenu(m);
                break;
              case 9:
                m=1;
                f=1;
                drawMenu(m);
                break;
            }
            break;
          case 3:  // MENU 3 - TASTI TELECOMANDO TOGGLE
            switch(f)
            {
              case 1:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,16,PSTR("          "));
                TV.print(50,16,results.value);
                irKeysToggle[0] = results.value;
                blinkLed(10);
                break;
              case 2:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,24,PSTR("          "));
                TV.print(50,24,results.value);
                irKeysToggle[1] = results.value;
                blinkLed(10);
                break;
              case 3:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,32,PSTR("          "));
                TV.print(50,32,results.value);
                irKeysToggle[2] = results.value;
                blinkLed(10);
                break;
              case 4:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,40,PSTR("          "));
                TV.print(50,40,results.value);
                irKeysToggle[3] = results.value;
                blinkLed(10);
                break;
              case 5:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,48,PSTR("          "));
                TV.print(50,48,results.value);
                irKeysToggle[4] = results.value;
                blinkLed(10);
                break;
              case 6:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,56,PSTR("          "));
                TV.print(50,56,results.value);
                irKeysToggle[5] = results.value;
                blinkLed(10);
                break;
              case 7:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,64,PSTR("          "));
                TV.print(50,64,results.value);
                irKeysToggle[6] = results.value;
                blinkLed(10);
                break;
              case 8:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,72,PSTR("          "));
                TV.print(50,72,results.value);
                irKeysToggle[7] = results.value;
                blinkLed(10);
                break;

              case 9:
                m=1;
                f=1;
                drawMenu(m);
                break;
            }
            break;
          case 4:  // MENU 4 - TASTI ON
             switch(f)
             {
              case 1:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,16,PSTR("        "));
                TV.print(50,16,results.value);
                irKeysOn[0] = results.value;
                blinkLed(10);
                break;
              case 2:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,24,PSTR("        "));
                TV.print(50,24,results.value);
                irKeysOn[1] = results.value;
                blinkLed(10);
                break;
              case 3:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,32,PSTR("        "));
                TV.print(50,32,results.value);
                irKeysOn[2] = results.value;
                blinkLed(10);
                break;
              case 4:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,40,PSTR("        "));
                TV.print(50,40,results.value);
                irKeysOn[3] = results.value;
                blinkLed(10);
                break;
              case 5:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,48,PSTR("        "));
                TV.print(50,48,results.value);
                irKeysOn[4] = results.value;
                blinkLed(10);
                break;
              case 6:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,56,PSTR("        "));
                TV.print(50,56,results.value);
                irKeysOn[5] = results.value;
                blinkLed(10);
                break;
              case 7:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,64,PSTR("        "));
                TV.print(50,64,results.value);
                irKeysOn[6] = results.value;
                blinkLed(10);
                break;
              case 8:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,72,PSTR("        "));
                TV.print(50,72,results.value);
                irKeysOn[7] = results.value;
                blinkLed(10);
                break;

               case 9:
                  m=1;
                  f=1;
                  drawMenu(m);
                  break;
            }
            break;
          case 5:  // MENU 5 - TASTI OFF
             switch(f)
             {
              case 1:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,16,PSTR("        "));
                TV.print(50,16,results.value);
                irKeysOff[0] = results.value;
                blinkLed(10);
                break;
              case 2:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,24,PSTR("        "));
                TV.print(50,24,results.value);
                irKeysOff[1] = results.value;
                blinkLed(10);
                break;
              case 3:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,32,PSTR("        "));
                TV.print(50,32,results.value);
                irKeysOff[2] = results.value;
                blinkLed(10);
                break;
              case 4:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,40,PSTR("        "));
                TV.print(50,40,results.value);
                irKeysOff[3] = results.value;
                blinkLed(10);
                break;
              case 5:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,48,PSTR("        "));
                TV.print(50,48,results.value);
                irKeysOff[4] = results.value;
                blinkLed(10);
                break;
              case 6:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,56,PSTR("        "));
                TV.print(50,56,results.value);
                irKeysOff[5] = results.value;
                blinkLed(10);
                break;
              case 7:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,64,PSTR("        "));
                TV.print(50,64,results.value);
                irKeysOff[6] = results.value;
                blinkLed(10);
                break;
              case 8:
                TV.stop();
                irrecv.enableIRIn();
                while(irrecv.decode(&results)==0); 
                TV.resume();
                TV.printPGM(50,72,PSTR("        "));
                TV.print(50,72,results.value);
                irKeysOff[7] = results.value;
                blinkLed(10);
                break;

                case 9:
                  m=1;
                  f=1;
                  drawMenu(m);
                  break;
            }
            break;
          case 6:  // MENU 6 - CONFIG GRUPPO 1
            switch(f)
            {
              case 1:
                group1[0] = !group1[0];
                if(group1[0]==1)
                  TV.print(50,16,"V");
                else
                  TV.print(50,16,"X");
                break;
              case 2:
                group1[1] = !group1[1];
                if(group1[1]==1)
                  TV.print(50,24,"V");
                else
                  TV.print(50,24,"X");
                break;
              case 3:
                group1[2] = !group1[2];
                if(group1[2]==1)
                  TV.print(50,32,"V");
                else
                  TV.print(50,32,"X");
                break;
              case 4:
                group1[3] = !group1[3];
                if(group1[3]==1)
                  TV.print(50,40,"V");
                else
                  TV.print(50,40,"X");
                break;
              case 5:
                group1[4] = !group1[4];
                if(group1[4]==1)
                  TV.print(50,48,"V");
                else
                  TV.print(50,48,"X");
                break;
              case 9:
                m=2;
                f=1;
                drawMenu(m);
                break;
            }
            break;
          case 7:  // MENU 5 - CONFIG GRUPPO 2
            switch(f)
            {
              case 1:
                group2[0] = !group2[0];
                if(group2[0]==1)
                  TV.printPGM(50,16,PSTR("V"));
                else
                  TV.printPGM(50,16,PSTR("X"));
                break;
              case 2:
                group2[1] = !group2[1];
                if(group2[1]==1)
                  TV.printPGM(50,24,PSTR("V"));
                else
                  TV.printPGM(50,24,PSTR("X"));
                break;
              case 3:
                group2[2] = !group2[2];
                if(group2[3]==1)
                  TV.printPGM(50,32,PSTR("V"));
                else
                  TV.printPGM(50,32,PSTR("X"));
                break;
              case 4:
                group2[3] = !group2[3];
                if(group2[3]==1)
                  TV.printPGM(50,40,PSTR("V"));
                else
                  TV.printPGM(50,40,PSTR("X"));
                break;
              case 5:
                group2[4] = !group2[4];
                if(group2[4]==1)
                  TV.printPGM(50,48,PSTR("V"));
                else
                  TV.printPGM(50,48,PSTR("X"));
                break;
              case 9:
                m=2;
                f=1;
                drawMenu(m);
                break;
            }          
            break;
          case 8:
            switch(f)
            { 
              case 1:
              case 2:
              case 3:
              case 4:
              case 5:
                TV.printPGM(100,8+restartMode*8,PSTR(" "));
                restartMode = f;
                TV.printPGM(100,8+restartMode*8,PSTR("V"));
                break;
              case 9:
                m=1;
                f=1;
                drawMenu(m);
                break;
            }
            break;
        }
        TV.delay(500);
      }
      if(key == 2)  //D3 -> SELECT
      {
        TV.print(0,16+(f-1)*8," ");
        f++;
        if(f>=10)
          f=1;
        TV.print(0,16+(f-1)*8,">");
        TV.delay(250);
      }
    }
}

void restoreSocketsStatus()
{
  switch(restartMode)
  {
    case 1:
      // LAST CONF
      loadLastSocketConf();
      for(byte port=0; port<5;port++)
      {
         if(pinStatus[port]==1)
         {
              digitalWrite(A0+port, HIGH);
         }
         else
         {
              digitalWrite(A0+port, LOW);
         }

      }
      break;
    case 2:
      // GROUP1 ON
      for(byte port=0; port<5;port++)
      {
         if(group1[port]==1)   // la porta fa parte del gruppo 1
         {
              digitalWrite(A0+port, HIGH);
              pinStatus[port]=1;
         }
      }
      break;
    case 3:
      // GROUP2 ON
      for(byte port=0; port<5;port++)
      {
         if(group2[port]==1)   // la porta fa parte del gruppo 2
         {
              digitalWrite(A0+port, HIGH);
              pinStatus[port]=1;
         }
      }
      break;
    case 4:
      // ALL ON
      for(byte port=0; port<5;port++)
      {
              digitalWrite(A0+port, HIGH);
              pinStatus[port]=1;
      }
      break;
    case 5:
    default:
      //ALL OFF
      break;
  }
}

void setup()
{  
  pinMode(A0, OUTPUT);
  pinMode(A1, OUTPUT);
  pinMode(A2, OUTPUT);
  pinMode(A3, OUTPUT);
  pinMode(A4, OUTPUT);
  digitalWrite(A0, LOW);
  digitalWrite(A1, LOW);
  digitalWrite(A2, LOW);
  digitalWrite(A3, LOW);
  digitalWrite(A4, LOW);

  pinMode(10, OUTPUT);
  digitalWrite(10,LOW);        // Led vicino al ricevitore IR spento

  pinMode(2,INPUT);
  digitalWrite(2,HIGH);        // Pullup tasto ENTER

  pinMode(3,INPUT);
  digitalWrite(3,HIGH);        // Pullup tasto SELECT

  readFromEEprom();            // Lettura dati configurazione dalla EEPROM
  restoreSocketsStatus();      // Ripristina lo stato delle prese dopo un blackout
  saveLastSocketConf();        // Salva lo stato
  blinkLed(10);                // Blink standard Led per indicare che il sistema è pronto

  if(TV.begin(PAL,120,90)==4)  // Attivazione buffer video, in caso di errore led fisso acceso
  {
    digitalWrite(10,HIGH);
    while(true);
  }

  TV.select_font(font4x6);  
  TV.printPGM(0,0,PSTR("iMULTIPRESA v1.0"));
  TV.printPGM(0,8,PSTR("PREMERE <ENTER>\nPER ATTIVARE IL MENU."));
  
  if(checkEnterMenu())     // Utente ha premuto ENTER entro il tempo
  {
    TV.printPGM(1,50,PSTR("ACCESSO AL MENU..."));
    TV.delay(1000);
    processMenu();
  }

  TV.stop();             // Interruzione TV out
  irrecv.enableIRIn();   // Attivazione degli IR
  blinkLed(10);          // Lampeggio LED per indicare l'attivazione degli IR  
}

void loop() 
{
  if(readKeys()==1)
  {
    blinkLed(10);          // Lampeggio LED per indicare l'attivazione del menu
    TV.resume();
    processMenu();
    TV.stop();             // Interruzione TV out
    irrecv.enableIRIn();   // Attivazione degli IR
    blinkLed(10);          // Lampeggio LED per indicare l'attivazione degli IR  
  }
  
  if (irrecv.decode(&results)) 
  {
    if(results.value == irKeysToggle[0])     // Toggle PRESA1
    {
        if(pinStatus[0]==0)
        {
          digitalWrite(A0, HIGH);
          pinStatus[0]=1;
          blinkLed(10);
        }
        else
        {
          digitalWrite(A0, LOW);
          pinStatus[0]=0;
          blinkLed(10,2,125,125);
        }
    }
    else if(results.value == irKeysToggle[1])     // Toggle PRESA2
    {
        if(pinStatus[1]==0)
        {
          digitalWrite(A1, HIGH);
          pinStatus[1]=1;
          blinkLed(10);
        }
        else
        {
          digitalWrite(A1, LOW);
          pinStatus[1]=0;
          blinkLed(10,2,125,125);
        }
    }
    else if(results.value == irKeysToggle[2])     // Toggle PRESA3
    {
        if(pinStatus[2]==0)
        {
          digitalWrite(A2, HIGH);
          pinStatus[2]=1;
          blinkLed(10);
        }
        else
        {
          digitalWrite(A2, LOW);
          pinStatus[2]=0;
          blinkLed(10,2,125,125);
        }
    }
    else if(results.value == irKeysToggle[3])     // Toggle PRESA4
    {
        if(pinStatus[3]==0)
        {
          digitalWrite(A3, HIGH);
          pinStatus[3]=1;
          blinkLed(10);
        }
        else
        {
          digitalWrite(A3, LOW);
          pinStatus[3]=0;
          blinkLed(10,2,125,125);
        }
    }
    else if(results.value == irKeysToggle[4])     // Toggle PRESA5
    {
        if(pinStatus[4]==0)
        {
          digitalWrite(A4, HIGH);
          pinStatus[4]=1;
          blinkLed(10);
        }
        else
        {
          digitalWrite(A4, LOW);
          pinStatus[4]=0;
          blinkLed(10,2,125,125);
        }
    }
    else if(results.value == irKeysToggle[5])     // Toggle GROUP1
    {
      for(byte port=0; port<5;port++)
      {
         if(group1[port]==1)   // la porta fa parte del gruppo 1
         {
            if(pinStatus[port]==0)
            {
              digitalWrite(A0+port, HIGH);
              pinStatus[port]=1;
              
            }
            else
            {
              digitalWrite(A0+port, LOW);
              pinStatus[port]=0;
              
            }           
         }
      }
      blinkLed(10,3,125,125);
    }
    else if(results.value == irKeysToggle[6])     // Toggle GROUP2
    {
      for(byte port=0; port<5;port++)
      {
         if(group2[port]==1)   // la porta fa parte del gruppo 2
         {
            if(pinStatus[port]==0)
            {
              digitalWrite(A0+port, HIGH);
              pinStatus[port]=1;
              
            }
            else
            {
              digitalWrite(A0+port, LOW);
              pinStatus[port]=0;
              
            }           
         }
      }
      blinkLed(10,3,125,125);      
    }
    else if(results.value == irKeysToggle[7])     // Toggle ALL
    {
      for(byte port=0; port<5;port++)
      {
            if(pinStatus[port]==0)
            {
              digitalWrite(A0+port, HIGH);
              pinStatus[port]=1;
              
            }
            else
            {
              digitalWrite(A0+port, LOW);
              pinStatus[port]=0;
              
            }           
         
      }
      blinkLed(10,3,125,125);        
    }

    else if(results.value == irKeysOn[0])     // On PRESA1
    {
          digitalWrite(A0, HIGH);
          pinStatus[0]=1;
          blinkLed(10);
    }
    else if(results.value == irKeysOn[1])     // On PRESA2
    {
          digitalWrite(A1, HIGH);
          pinStatus[1]=1;
          blinkLed(10);
    }
    else if(results.value == irKeysOn[2])     // On PRESA3
    {
          digitalWrite(A2, HIGH);
          pinStatus[2]=1;
          blinkLed(10);
    }
    else if(results.value == irKeysOn[3])     // On PRESA4
    {
          digitalWrite(A3, HIGH);
          pinStatus[3]=1;
          blinkLed(10);
    }
    else if(results.value == irKeysOn[4])     // On PRESA5
    {
          digitalWrite(A4, HIGH);
          pinStatus[4]=1;
          blinkLed(10);
    }
    else if(results.value == irKeysOn[5])     // On GROUP1
    {
      for(byte port=0; port<5;port++)
      {
         if(group1[port]==1)   // la porta fa parte del gruppo 1
         {
              digitalWrite(A0+port, HIGH);
              pinStatus[port]=1;
         }
      }
      blinkLed(10);
    }
    else if(results.value == irKeysOn[6])     // On GROUP2
    {
      for(byte port=0; port<5;port++)
      {
         if(group2[port]==1)   // la porta fa parte del gruppo 2
         {
              digitalWrite(A0+port, HIGH);
              pinStatus[port]=1;
         }
      }
      blinkLed(10);      
    }
    else if(results.value == irKeysOn[7])     // On ALL
    {
      for(byte port=0; port<5;port++)
      {
              digitalWrite(A0+port, HIGH);
              pinStatus[port]=1;
      }
      blinkLed(10);        
    }

    else if(results.value == irKeysOff[0])     // Off PRESA1
    {
          digitalWrite(A0, LOW);
          pinStatus[0]=0;
          blinkLed(10,2,125,125);
    }
    else if(results.value == irKeysOff[1])     // Off PRESA2
    {
          digitalWrite(A1, LOW);
          pinStatus[1]=0;
          blinkLed(10,2,125,125);
    }
    else if(results.value == irKeysOff[2])     // Off PRESA3
    {
          digitalWrite(A2, LOW);
          pinStatus[2]=0;
          blinkLed(10,2,125,125);
    }
    else if(results.value == irKeysOff[3])     // Off PRESA4
    {
          digitalWrite(A3, LOW);
          pinStatus[3]=0;
          blinkLed(10,2,125,125);
    }
    else if(results.value == irKeysOff[4])     // Off PRESA5
    {
          digitalWrite(A4, LOW);
          pinStatus[4]=0;
          blinkLed(10,2,125,125);
    }
    else if(results.value == irKeysOff[5])     // Off GROUP1
    {
      for(byte port=0; port<5;port++)
      {
         if(group1[port]==1)   // la porta fa parte del gruppo 1
         {
              digitalWrite(A0+port, LOW);
              pinStatus[port]=0;
         }
      }
      blinkLed(10,2,125,125);
    }
    else if(results.value == irKeysOff[6])     // Toggle GROUP2
    {
      for(byte port=0; port<5;port++)
      {
         if(group2[port]==1)   // la porta fa parte del gruppo 2
         {
              digitalWrite(A0+port, LOW);
              pinStatus[port]=0;
         }
      }
      blinkLed(10,2,125,125);      
    }
    else if(results.value == irKeysOff[7])     // Off ALL
    {
      for(byte port=0; port<5;port++)
      {
              digitalWrite(A0+port, LOW);
              pinStatus[port]=0;
      }
      blinkLed(10,2,125,125);        
    }

    if(restartMode == 1)       // Salva lo stato delle prese se la modalità ripristino è impostata a "ultima configurazione"
      saveLastSocketConf();

    irrecv.resume(); // Receive the next value
  }
}



